<?php
/**
 * @file
 * Provide Behat step-definitions for generic OA tests.
 *
 */

use Drupal\DrupalExtension\Context\DrupalSubContextInterface;
use Behat\Behat\Context\BehatContext;
use Drupal\Component\Utility\Random;
use Behat\Behat\Event\StepEvent;
use Behat\Mink\Exception\ElementNotFoundException;


// To take screenshot: $this->getMainContext()->showLastResponse();

class TestOaSubContext extends BehatContext implements DrupalSubContextInterface {

  /**
   * Holds a list of temporary nodes created by our steps.
   *
   * @var array
   */
  protected $nodes = array();

  // Keeps track of the existing comment settings for a content type.
  protected $comment_settings = array();
  protected $comment_paragraph_settings = array();

  /**
   * Initializes context.
   */
  public function __construct(array $parameters = array()) {
  }

  public static function getAlias() {
    return 'testoa';
  }


  /**
   * @AfterScenario
   *
   * Clean up nodes that were created during testing
   */
  public function removeNodes() {
    if (!empty($this->nodes)){
      foreach ($this->nodes as $node) {
        node_delete($node->nid); //$this->getMainContext()->getDriver()->nodeDelete($node);
      }
      $this->nodes = array();
    }
  }

  /**
   * Get the session from the parent context.
   */
  protected function getSession() {
    return $this->getMainContext()->getSession();
  }

  /**
   * @Then /^I uncheck permission "([^"]*)" for "([^"]*)"$/
   * @Then /^I check permission "([^"]*)" for "([^"]*)"$/
   */
  public function iUncheckPermissionFor($permission, $role) {
    $driver = $this->getSession()->getDriver();
    $header_rows = $driver->find("//table[@id='permissions']/thead/tr/th");
    $found = FALSE;
    foreach ($header_rows as $pos => $header_row) {
      if ($header_row->getText() == $role) {
        $found = TRUE;
        $pos++;
        break;
      }
    }
    if ($found) {
      $checkbox = $driver->find("//table[@id='permissions']//td[contains(concat(' ', @class, ' '), ' permission ')]/div[contains(string(.), '" . $permission . "')]/../../td[" . $pos . "]//input[@type='checkbox']");
      if ($checkbox) {
        $checkbox[0]->click();
      }
      else {
        throw new \Exception(sprintf('Unable to find checkbox for %s on page %s', $permission, $this->getSession()->getCurrentUrl()));
      }
    }
    else {
      throw new \Exception(sprintf('Unable to find column for role %s on page %s', $role, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I click the checkbox in the table row containing "([^"]*)"$/
   */
  public function iClickTheCheckboxInTheTableRowContaining($text) {
    $driver = $this->getSession()->getDriver();
    $checkbox = $driver->find("//tr[contains(string(.), '" . $text . "')]//input[@type='checkbox']");
    if ($checkbox) {
      $checkbox[0]->click();
    }
    else {
      throw new \Exception(sprintf('Unable to find the checkbox with %s on %s', $text, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I should see the text "([^"]*)" in the table row containing "([^"]*)"$/
   */
  public function iShouldSeeTheTextInTheTableRowContaining($text1, $text2) {
    $driver = $this->getSession()->getDriver();
    $row = $driver->find("//tr[contains(string(.), '" . $text1 . "') and contains(string(.), '" . $text2 . "')]");
    if (!$row) {
      throw new \Exception(sprintf('Unable to find the row %s and %s on %s', $text1, $text2, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I should see the text "([^"]*)" under "([^"]*)"$/
   */
  public function iShouldSeeTheTextUnder($text, $selector) {
    $page = $this->getSession()->getPage();
    $row = $page->find('css', $selector);
    if ($row && ($row_text = $row->getText())) {
      if (strpos($row_text, $text) === FALSE) {
        throw new \Exception(sprintf('Unable to find text %s for selector %s on page %s', $text, $selector, $this->getSession()->getCurrentUrl()));
      }
    }
    else {
      throw new \Exception(sprintf('Unable to find text for selector %s on page %s', $selector, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I should not see the text "([^"]*)" under "([^"]*)"$/
   */
  public function iShouldNotSeeTheTextUnder($text, $selector) {
    $page = $this->getSession()->getPage();
    $row = $page->find('css', $selector);
    if ($row && ($row_text = $row->getText())) {
      if (strpos($row_text, $text) !== FALSE) {
        throw new \Exception(sprintf('Able to find text %s for selector %s on page %s', $text, $selector, $this->getSession()->getCurrentUrl()));
      }
    }
  }

  /**
   * @Given /^I click to add pane "([^"]*)"$/
   */
  public function iClickToAddPane($app_title) {
    $driver = $this->getSession()->getDriver();

    $link = $driver->find("//div[contains(concat(' ', @class, ' '), ' panel-title ')][text()='" . $app_title  . "']/../a[text()=' Add']");
    if (!$link) {
      // Handle new Single preview mode
      $link = $driver->find("//a[contains(concat(' ', @class, ' '), ' button ')][text()='" . $app_title  . "']/../a[text()=' Add']");
    }
    if ($link) {
      $link[0]->click();
    }
    else {
      throw new \Exception(sprintf('The app %s not found on to click %s', $app_title, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Given /^I click "([^"]*)" for the pane that contains class "([^"]*)"$/
   */
  public function iClickForThePaneThatContainsClass($link, $class) {
    $driver = $this->getSession()->getDriver();
    $handlebar = $driver->find("//div[contains(concat(' ', @class, ' '), ' " . $class . " ')]/ancestor::div[contains(concat(' ', @class, ' '), ' panels-ipe-portlet-wrapper ')]//div[contains(concat(' ', @class, ' '), ' panels-ipe-dragbar ')]");
    if ($handlebar) {
      $handlebar[0]->clickLink($link);
    }
    else {
      throw new \Exception(sprintf('The pane containing class %s not found on page %s', $class, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Given /^I select "([^"]*)" from sitemap dropdown$/
   */
  public function iSelectFromSitemapDropdown($space) {
    $driver = $this->getSession()->getDriver();

    $button = $driver->find("//button[contains(concat(' ', @class, ' '), ' oa-sitemap-search-toggle ')]");
    $button[0]->click();
    $switch = $driver->find("//div[contains(concat(' ', @class, ' '), ' oa-sitemap-search ')]/ul/li/a[contains(normalize-space(text()),'" . $space . "')]");
    if (!empty($switch[0])) {
      $switch[0]->click();
    }
    else {
      throw new \Exception(sprintf('Unable to find space %s in sitemap dropdown on page %s', $space, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Given /^I select top level from sitemap dropdown$/
   */
  public function iSelectTopLevelFromSitemapDropdown() {
    $driver = $this->getSession()->getDriver();

    $button = $driver->find("//ul[contains(concat(' ', @class, ' '), ' oa-sitemap-breadcrumbs ')]/li/a[@href='#/space/0']");
    $button[0]->click();
  }

  /**
   * @When /^I visit "([^"]*)" in space "([^"]*)"$/
   * @When /^I visit "([^"]*)" in space$/
   */
  public function iVisitInSpace($path, $title = NULL) {
    $node = $this->getSpace($title);
    $this->getSession()->visit($this->getMainContext()->locatePath('node/' . $node->nid));
    $this->getSession()->visit($this->getMainContext()->locatePath($path));
  }

  /**
   * @When /^I visit content "([^"]*)" in space "([^"]*)"$/
   */
  public function iVisitContentInSpace($content_title, $space_title) {
    if ($nid = $this->getNidFromTitleForSpace($content_title, $space_title)) {
      $this->getSession()->visit($this->getMainContext()->locatePath('node/' . $nid));
    }
  }
  /**
   * @When /^I visit the homepage in private space$/
   * @When /^I visit the homepage in private space "([^"]*)"$/
   */
  public function iVisitTheHomepageInPrivateSpace($title = NULL) {
    $node = $this->getSpace($title, TRUE, array('group_access' => 1));
    $this->getSession()->visit($this->getMainContext()->locatePath('node/' . $node->nid));
  }

  /**
   * @When /^I visit the homepage in space$/
   * @When /^I visit the homepage in space "([^"]*)"$/
   */
  public function iVisitTheHomepageInSpace($title = NULL) {
    $node = $this->getSpace($title);
    $this->getSession()->visit($this->getMainContext()->locatePath('node/' . $node->nid));
  }

  /**
   * @Then /^I visit the current space homepage$/
   */
  public function iVisitTheCurrentSpaceHomepage() {
    $driver = $this->getSession()->getDriver();
    $links = $driver->find("//a[@id='oa-breadcrumb-space-link']");
    if ($links) {
      $links[0]->click();
    }
    else {
      throw new \Exception(sprintf('Unable to find the current space link on page %s', $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I visit the current section homepage$/
   */
  public function iVisitTheCurrentSectionHomepage() {
    $driver = $this->getSession()->getDriver();
    $links = $driver->find("//a[@id='oa-breadcrumb-section-link']");
    if ($links) {
      $links[0]->click();
    }
    else {
      throw new \Exception(sprintf('Unable to find the current section homepage on the page %s', $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I log in with a member of "([^"]*)"$/
   * @Then /^I log in with member "([^"]*)" of "([^"]*)"$/
   */
  public function ILogInWithAMemberOfSpace($name, $title = NULL) {
    if (!$title) {
      $title = $name;
      $name = NULL;
    }
    $user = $name ? user_load_by_name($name) : NULL;

    if (!$user) {
      $random = new Random;
      // Create user (and project)
      $user = (object) array(
        'name' => $name ? $name : $random->name(8),
        'pass' => $random->name(16),
        'role' => 'authenticated user',
      );
      $user->mail = "{$user->name}@example.com";

      // Create a new user.
      $this->getMainContext()->getDriver()->userCreate($user);
      $this->users[$user->name] = $this->user = $user;
    }
    if ($title) {
      // This space should likely exist already.
      $node = $this->getSpace($title);
      // Add user to space.
      og_group('node', $node->nid, array('entity' => $user));
    }
    return $this->loginUser($user);
  }

  /**
   * @Then /^I log in with the owner of space "([^"]*)"$/
   */
  public function iLogInWithTheOwnerOfSpace($title) {
    if (($node = $this->getSpace($title)) && $node->uid && ($account = user_load($node->uid))) {
      return $this->loginUser($account);
    }
    else {
      throw new \Exception(sprintf('Unable to find group owner of %s on %s', $title, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I have a member "([^"]*)" of "([^"]*)"$/
   */
  public function iHaveAMemberOf($name, $title) {
    $account = $name ? user_load_by_name($name) : NULL;

    if (!$account) {
      $random = new Random;
      // Create user (and project)
      $account = (object) array(
        'name' => $name ? $name : $random->name(8),
        'pass' => $random->name(16),
        'role' => 'authenticated user',
      );
      $account->mail = "{$account->name}@example.com";

      // Create a new user.
      $this->getMainContext()->getDriver()->userCreate($account);
      $this->users[$account->name] = $this->user = $account;
    }
    if ($title) {
      // This space should likely exist already.
      $node = $this->getSpace($title);
      // Add user to space.
      og_group('node', $node->nid, array('entity' => $account));
    }
  }


  /**
   * @Then /^the "([^"]*)" select box should have option "([^"]*)"$/
   */
  public function theSelectBoxShouldHaveOption($identifier, $option_text) {
    $select = $this->getSession()->getPage()->findField($identifier);
    if (empty($select)) {
      throw new \Exception(sprintf('We couldn\'nt find "%s" on the page', $identifier));
    }
    $option = $select->findAll('xpath', "//option[text()='" . $option_text . "']");
    if ($option) {
      return;
    }
    else {
      throw new \Exception(sprintf('Unable to find the option %s for %s on page %s', $option_text, $identifier, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^the "([^"]*)" select box should not have option "([^"]*)"$/
   */
  public function theSelectBoxShouldNotHaveOption($identifier, $option_text) {
    $select = $this->getSession()->getPage()->findField($identifier);
    if (empty($select)) {
      throw new \Exception(sprintf('We couldn\'nt find "%s" on the page', $identifier));
    }
    $option = $select->findAll('xpath', "//option[text()='" . $option_text . "']");
    if (!$option) {
      return;
    }
    else {
      throw new \Exception(sprintf('Able to find the option %s for %s on page %s', $option_text, $identifier, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * This is broken but may be fixed in behat2. //@When /^I hover over the link "([^"]*)"$/
   */
  public function iHoverOverTheElement($link) {
    $element = $this->getSession()->getPage()->findLink($link); // runs the actual query and returns the element

    // errors must not pass silently
    if (!$element) {
      throw new \InvalidArgumentException(sprintf('Could not find link: "%s"', $link));
    }

    // ok, let's hover it
    $element->mouseOver();
  }

  /**
   * @Then /^I log in with "([^"]*)"$/
   */
  public function ILogInWith($name) {
    $user = $name ? user_load_by_name($name) : NULL;
    if (!$user) {
      $random = new Random;
      // Create user (and project)
      $user = (object) array(
        'name' => $name ? $name : $random->name(8),
        'pass' => $random->name(16),
        'role' => 'authenticated user',
      );
      $user->mail = "{$user->name}@example.com";

      // Create a new user.
      $this->getMainContext()->getDriver()->userCreate($user);
      $this->users[$user->name] = $this->user = $user;
    }
    return $this->loginUser($user);
  }

  function loginUser($user) {
    if ($this->getMainContext()->loggedIn()) {
      $this->getMainContext()->logOut();
    }
    $base_url = rtrim($this->getMainContext()->locatePath('/'), '/');
    $login_link = $this->getMainContext()->getDriver('drush')->drush('uli', array(
      "'$user->name'",
      '--browser=0',
      "--uri=${base_url}",
    ));
    // Trim EOL characters. Required or else visiting the link won't work.
    $login_link = trim($login_link);
    $this->getSession()->visit($this->getMainContext()->locatePath($login_link));
    return TRUE;
  }


  /**
   * Creates and authenticates a user with the given role via Drush.
   *
   * @Given /^I belong to space "(?P<space>[^"]*)"$/
   */
  public function assertUserPartOfSpace($space) {
    // Add user to a space
    $user = user_load($this->getMainContext()->user->uid);
    $node = $this->getSpace($space);
    og_group('node', $node->nid, array('entity' => $user));
    return $user && $node;
  }


  /**
   * Create a new space.
   *
   * @param string $title
   *   The internal machine name for the new Page.
   */
  protected function getSpace($title = NULL, $create = TRUE, $fields = array()) {
    $user = user_load($this->getMainContext()->user->uid);
    if (!$title) {
      $random = new Random();
      $title = $random->name(8);
    }
    else {
      foreach ($this->nodes as $node) {
        if ($node->title == $title) {
          return $node;
        }
      }
    }
    if ($nodes = node_load_multiple(array(), array('title' => $title, 'type' => 'oa_space'))) {
      return reset($nodes);
    }
    if (!$create) {
      return FALSE;
    }
    $node = array(
      'title' => $title,
      'uid' => $user ? $user->uid : 1,
      'type' => 'oa_space',
      'language' => LANGUAGE_NONE,
    ) + $fields;
    $node = (object) $node;
    $node = $this->getMainContext()->getDriver()->createNode($node);
    $this->nodes[$node->nid] = $node;
    return $node;
  }

  /**
   * Take screenshot when step fails. Works only with Selenium2Driver.
   *
   * If on a Mac, runnining from the command line, with the SHOW_SNAPSHOT
   *  environment variable set to 1, the snapshot will open in Preview.
   *
   * Screenshot is saved at [Date]/[Feature]/[Scenario]/[Step].jpg
   *
   * @see http://classically.me/blogs/selenium-screenshot-behat-scenario-step-failure.
   *
   * @AfterStep @javascript
   */
  public function takeScreenshotAfterFailedStep(Behat\Behat\Event\StepEvent $event) {
    if ($event->getResult() === Behat\Behat\Event\StepEvent::FAILED) {
      $driver = $this->getSession()->getDriver();
      if ($driver instanceof Behat\Mink\Driver\Selenium2Driver) {
        $step = $event->getStep();
        $path = array(
          'date' => date("Ymd-Hi"),
          'feature' => $step->getParent()->getFeature()->getTitle(),
          'scenario' => $step->getParent()->getTitle(),
          'step' => $step->getType() . ' ' . $step->getText(),
        );
        $path = preg_replace('/[^\-\.\w]/', '_', $path);
        $filename = '/tmp/behat/' .  implode('/', $path) . '.jpg';

        // Create directories if needed.
        if (!@is_dir(dirname($filename))) {
          @mkdir(dirname($filename), 0775, TRUE);
        }

        file_put_contents($filename, $driver->getScreenshot());
        if (PHP_OS === "Darwin" && (bool) getenv('SHOW_SNAPSHOT') === TRUE && PHP_SAPI === "cli") {
          exec('open -a "Preview.app" ' . $filename);
        }
      }
    }
  }

  /**
   * @Given /^I click "(?P<link>(?:[^"]|\\")*)" for comment "([^"]*)"$/
   */
  public function IClickForComment($link, $row_number) {
    $driver = $this->getSession()->getDriver();
    $rows = $driver->find("//div[contains(concat(' ', @class, ' '), ' oa-comment ')]");
    $row_number--;
    if (!empty($rows[$row_number])) {
      $rows[$row_number]->clickLink($link);
    }
    else {
      throw new \Exception(sprintf('Unable to find the comment %s on %s', $row_number, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I should see the link "(?P<link>(?:[^"]|\\")*)" for comment "([^"]*)"$/
   */
  public function IShouldSeeTheLinkForComment($link, $row_number) {
    $driver = $this->getSession()->getDriver();
    $rows = $driver->find("//div[contains(concat(' ', @class, ' '), ' oa-comment ')]");
    $row_number--;
    if (!empty($rows[$row_number])) {
      $result = $rows[$row_number]->findLink($link);
      try {
        if ($result && !$result->isVisible()) {
          throw new \Exception(sprintf("No visible link to '%s' for comment %s on the page %s", $link, $row_number, $this->getSession()->getCurrentUrl()));
        }
      }
      catch (UnsupportedDriverActionException $e) {
        // We catch the UnsupportedDriverActionException exception in case
        // this step is not being performed by a driver that supports javascript.
        // All other exceptions are valid.
      }

      if (empty($result)) {
        throw new \Exception(sprintf("No link to '%s' for comment %s on the page %s", $link, $row_number, $this->getSession()->getCurrentUrl()));
      }
    }
    else {
      throw new \Exception(sprintf('Unable to find the comment %s on %s', $row_number, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I should see the text "([^"]*)" for comment "([^"]*)"$/
   */
  public function IShouldSeeTheTextForComment($text_to_find, $row_number) {
    $driver = $this->getSession()->getDriver();
    $rows = $driver->find("//div[contains(concat(' ', @class, ' '), ' oa-comment ')]");
    $row_number--;
    if (!empty($rows[$row_number])) {
      $text = $rows[$row_number]->getText();
      if (strpos($text, $text_to_find) === FALSE) {
        throw new \Exception(sprintf("No text '%s' in comment %s on the page %s", $text_to_find, $row_number, $this->getSession()->getCurrentUrl()));
      }
    }
    else {
      throw new \Exception(sprintf('Unable to find the comment %s on %s', $row_number, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^the link "([^"]*)" should exist$/
   */
  public function theLinkShouldExist($link) {
    $element = $this->getSession()->getPage()->findLink($link);
    if (!$element) {
      throw new \Exception(sprintf('Unable to find the link %s on %s', $link, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^the link "([^"]*)" should not exist$/
   */
  public function theLinkShouldNotExist($link) {
    $element = $this->getSession()->getPage()->findLink($link);
    if ($element) {
      throw new \Exception(sprintf('Able to find the link %s on %s', $link, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^the title of the page should contain "([^"]*)"$/
   */
  public function theTitleOfThePageShouldContain($text) {
    $driver = $this->getSession()->getDriver();
    $title = $driver->find("//title[contains(normalize-space(text()), '"  . $text . "')]");
    if (!$title) {
      throw new \Exception(sprintf('Unable to find title tag containing %s on the page %s', $text, $this->getSession()->getCurrentUrl()));
    }
  }


  /**
   * @Given /^I click link containing "(?P<link>(?:[^"]|\\")*)"$/
   */
  public function iClickLinkContaining($link) {
    $driver = $this->getSession()->getDriver();
    $links = $driver->find("//a[contains(normalize-space(text()), '"  . $link . "')]");
    if ($links) {
      $links[0]->click();
    }
    else {
      throw new \Exception(sprintf('Unable to find link containing %s on the page %s', $link, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Given /^I select "([^"]*)" from the dropdown results$/
   */
  public function iSelectFromTheDropdownResults($text) {
    $driver = $this->getSession()->getDriver();
    // behat on intergration doesn't like finding this, so retry till it's there.
    for ($i = 0; $i < 10; $i++) {
      $links = $driver->find("//div[@id='select2-drop' or @class='chosen-drop' or @id='autocomplete']//ul//*[contains(normalize-space(text()), '"  . $text . "')]");
      if ($links) {
        break;
      }
      sleep(2);
      $this->getSession()->wait(5000, 'typeof jQuery !== "undefined" && jQuery.active === 0');
    }
    if ($links) {
      $links[0]->click();
    }
    else {
      throw new \Exception(sprintf('Unable to find the drop down results containing %s on the page %s', $text, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^I fill the select2 input for "([^"]*)" with "([^"]*)"$/
   */
  public function iFillTheSelectInputForWith($class, $value) {
    $driver = $this->getSession()->getDriver();
    $input = $driver->find("//div[contains(concat(' ', @class, ' '), ' " . $class . " ')]//input[contains(concat(' ', @class, ' '), ' select2-input ')]");
    if ($input) {
      $input[0]->setValue($value);
    }
    else {
      throw new \Exception(sprintf('The select input under class %s not found on page %s', $class, $this->getSession()->getCurrentUrl()));
    }
  }


  /**
   * @Then /^I should see the image "([^"]*)"$/
   */
  public function shouldSeeTheImage($image_name) {
    $driver = $this->getSession()->getDriver();
    $images = $driver->find("//img");
    if ($images) {
      foreach ($images as $image) {
        if ($src = $image->getAttribute('src')) {
          $subpart = substr($image_name, 0, strpos($image_name, '.'));
          if (strpos($src, $subpart) !== FALSE) {
            return;
          }
        }
      }
    }
    throw new \Exception(sprintf('Unable to find image %s at %s', $image_name, $this->getSession()->getCurrentUrl()));
  }

  /**
   * @Then /^the banner image should be "([^"]*)"$/
   */
  public function theBannerImageShouldBe($image_name) {
    $driver = $this->getSession()->getDriver();
    $image = $driver->find("//img[contains(concat(' ', @class, ' '), ' oa-banner-overlay-img ')]");
    if ($image) {
      if ($src = $image[0]->getAttribute('src')) {
        $subpart = substr($image_name, 0, strpos($image_name, '.'));
        if (strpos($src, $subpart) === FALSE) {
          throw new \Exception(sprintf('Banner source image is  %s instead of %s on %s', $src, $image_name, $this->getSession()->getCurrentUrl()));
        }
      }
      else {
        throw new \Exception(sprintf('Banner source image does not have a src on %s', $this->getSession()->getCurrentUrl()));
      }
    }
    else {
      throw new \Exception(sprintf('Unable to find banner image at %s', $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^the banner image should not be visible$/
   */
  public function theBannerImageShouldNotBeVisible() {
    $driver = $this->getSession()->getDriver();
    $image = $driver->find("//img[contains(concat(' ', @class, ' '), ' oa-banner-overlay-img ')]");
    if ($image) {
      throw new \Exception(sprintf('Banner image found at %s', $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^the banner image should not be "([^"]*)"$/
   */
  public function theBannerImageShouldNotBe($image_name) {
    $driver = $this->getSession()->getDriver();
    $image = $driver->find("//img[contains(concat(' ', @class, ' '), ' oa-banner-overlay-img ')]");
    if ($image) {
      if ($src = $image[0]->getAttribute('src')) {
        $subpart = substr($image_name, 0, strpos($image_name, '.'));
        if (strpos($src, $subpart) !== FALSE) {
          throw new \Exception(sprintf('Banner source image is  %s when it should not be %s on %s', $src, $image_name, $this->getSession()->getCurrentUrl()));
        }
      }
      else {
        throw new \Exception(sprintf('Banner source image does not have a src on %s', $this->getSession()->getCurrentUrl()));
      }
    }
    else {
      throw new \Exception(sprintf('Unable to find banner image at %s', $this->getSession()->getCurrentUrl()));
    }
  }


  /**
   * @Then /^the space link should be "([^"]*)"$/
   */
  public function theSpaceLinkShouldBe($text) {
    $driver = $this->getSession()->getDriver();
    $link = $driver->find("//a[@id='oa-breadcrumb-space-link' and normalize-space(text())='" . $text . "']");
    if (!$link) {
      throw new \Exception(sprintf('The space link is not %s at %s', $text, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Then /^the space parent link should be "([^"]*)"$/
   */
  public function theSpaceParentLinkShouldBe($text) {
    $driver = $this->getSession()->getDriver();
    $link = $driver->find("//a[@id='oa-breadcrumb-parent-link' and normalize-space(text())='" . $text . "']");
    if (!$link) {
      throw new \Exception(sprintf('The parent space link is not %s at %s', $text, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @then /^I finish the document creation$/
   * @then /^I finish the file creation$/
   * @then /^I finish the image creation$/
   */
  public function IFinishTheDocumentCreation() {
    $driver = $this->getSession()->getDriver();
    $button = $driver->find("//input[@id='edit-create' or @id='edit-submit' or @id='edit-next']");
    if ($button) {
      $button[0]->click();
      $this->getSession()->switchToIFrame();
    }
    else {
      throw new \Exception(sprintf('The create file button not found on %s', $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @then /^I go to next document creation step$/
   */
  public function IGoToNextDocumentCreationStep() {
    $driver = $this->getSession()->getDriver();
    $button = $driver->find("//input[@id='edit-create' or @id='edit-submit' or @id='edit-next']");
    if ($button) {
      $button[0]->click();
      $this->getSession()->switchToIFrame();
      $this->getSession()->switchToIFrame("oaFilesUpload");
    }
    else {
      throw new \Exception(sprintf('The save button not found on %s', $this->getSession()->getCurrentUrl()));
    }
  }

  private $conf_variables = array();
  /**
   * @Then /^I store the "([^"]*)" variable$/
   */
  public function iStoreTheVariable($variable_name) {
    global $conf;
    if (array_key_exists($variable_name, $conf)) {
      $this->conf_variables[$variable_name] = $conf[$variable_name];
    }
  }

  /**
   * @AfterScenario
   *
   * reset variables.
   */
  public function resetConfVariables() {
    foreach ($this->conf_variables as $variable_name => $value) {
      variable_set($variable_name, $value);
    }
  }

  /**
   * @When /^comments are enabled for "([^"]*)"$/
   */
  public function commentsAreEnabledFor($content_type) {
    $this->setCommentSettngs($content_type, COMMENT_NODE_OPEN);
  }

  /**
   * @When /^comments are disabled for "([^"]*)"$/
   */
  public function commentsAreDisabledFor($content_type) {
    $this->setCommentSettngs($content_type, COMMENT_NODE_HIDDEN);
  }

  /**
   * @When /^paragraphs on comments are enabled for "([^"]*)"$/
   */
  public function commentsParagraphsAreEnabledFor($content_type) {
    $this->setCommentParagraphSettngs($content_type, TRUE);
  }

  /**
   * @When /^paragraphs on comments are disabled for "([^"]*)"$/
   */
  public function commentsParagraphsAreDisabledFor($content_type) {
    $this->setCommentParagraphSettngs($content_type, FALSE);
  }

  /**
   * @Given /^I select media item "([^"]*)"$/
   */
  public function iSelectMediaItem($arg1) {
    $driver = $this->getSession()->getDriver();
    $links = $driver->find("//div[@class='media-item' and contains(@title,'" . $arg1  . "')]");
    if ($links) {
      foreach ($links as $link) {
        if ($link->isVisible()) {
          $link->click();
          $buttons = $driver->find("//a[contains(concat(' ', @class, ' '), ' button ') and text()='Submit']");
          foreach ($buttons as $button) {
            if ($button->isVisible()) {
              $button->click();
              $this->getSession()->switchToIFrame();
              return;
            }
          }
        }
      }
    }
    throw new \Exception(sprintf('Unable to find a visible media item with name %s on %s', $arg1, $this->getSession()->getCurrentUrl()));
  }

  /**
   * @Given /^I check the box to create a new document$/
   */
  public function iCheckTheBoxtoCreateANewDocument() {
    $driver = $this->getSession()->getDriver();
    $links = $driver->find("//input[@type='checkbox' and contains(@name,'update')]");
    if ($links) {
      $links[0]->click();
    }
    else {
      throw new \Exception(sprintf('Unable to check the update checkbox on %s', $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * @Given /^I attach the file "([^"]*)" via plupload$/
   */
  public function iAttachTheFileViaPlupload($path) {
    if ($this->getMainContext()->getMinkParameter('files_path')) {
      $fullPath = rtrim(realpath($this->getMainContext()->getMinkParameter('files_path')), DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.$path;
      if (is_file($fullPath)) {
        $path = $fullPath;
      }
    }
    $driver = $this->getSession()->getDriver();
    $links = $driver->find("//div[contains(concat(' ', @class, ' '), ' plupload ')]/input[@type='file' and @multiple='multiple']");
    if ($links) {
      $links[0]->attachFile($path);
      $links = $driver->find("//a[contains(concat(' ', @class, ' '), ' plupload_start ')]");
      $links[0]->click();
    }
    else {
      throw new \Exception(sprintf('Unable to find the file input element on %s', $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * Get NID of content in given space.
   */
  function getNidForContentInSpace($content_title, $space) {
    return db_query("SELECT nid FROM {node} node INNER JOIN {og_membership} og on og.entity_type = 'node' and og.etid = nid AND og.group_type = 'node' WHERE node.title = :title AND og.gid = :gid", array(':title' => $content_title, ':gid' => $space->nid))->fetchColumn();
  }

  /**
   * Transforms Title of content in a space to NID.
   *
   * @Transform /^NID of '([A-Za-z0-9_ :@]+)' in '([A-Za-z0-9_ :@]+)'$/
   */
  public function getNidFromTitleForSpace($content_title, $space_title) {
    $content_title = $this->getStaticTimestamp($content_title);
    $space_title = $this->getStaticTimestamp($space_title);
    $node = $this->getSpace($space_title, FALSE);
    if ($node) {
      $nid = $this->getNidForContentInSpace($content_title, $node);
      if (!$nid) {
        throw new \Exception(sprintf('The content with title %s was not found for space %s on the page %s', $content_title, $space_title, $this->getSession()->getCurrentUrl()));
      }
      return $nid;
    }
    else {
      throw new \Exception(sprintf('The space with title %s was not found on the page %s', $space_title, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * Transforms Title of content in a space to node.
   *
   * Does NOT error out when cannot find.
   *
   * @Transform /^content '([A-Za-z0-9_ :@]+)' in '([A-Za-z0-9_ :@]+)'$/
   */
  public function getNodeFromTitleForSpace($content_title, $space_title) {
    $content_title = $this->getStaticTimestamp($content_title);
    $space_title = $this->getStaticTimestamp($space_title);
    $node = $this->getSpace($space_title, FALSE);
    if ($node) {
      if ($nid = $this->getNidForContentInSpace($content_title, $node)) {
        return node_load($nid);
      }
    }
    return FALSE;
  }

  /**
   * Get the term tid.
   *
   * @Transform /\[tid:([A-Za-z0-9_ :@]+)\]/
   */
  public function getTermTid($name) {
    $name = $this->getStaticTimestamp($name);
    if ($terms = taxonomy_get_term_by_name($name)) {
      $term = reset($terms);
      return $term->tid;
    }
  }

  /**
   * Get the create space link.
   *
   * @Transform /^create space url for '([A-Za-z0-9_ :@]+)'$/
   */
  public function getCreateSpaceLink($name) {
    $name = $this->getStaticTimestamp($name);
    if ($terms = taxonomy_get_term_by_name($name)) {
      $term = reset($terms);
      return 'node/add/oa-space/' . $term->tid;
    }
  }



  /**
   * @Then /^content "([^"]*)" should exist$/
   */
  public function contentShouldExist($node) {
    if ($node) {
      return;
    }
    else {
      throw new \Exception(sprintf('The content does not exist'));
    }
  }

  /**
   * @Then /^content "([^"]*)" should not exist$/
   */
  public function contentShouldNotExist($node) {
    if (!$node) {
      return;
    }
    else {
      throw new \Exception(sprintf('The content %s does exist', $node->title));
    }
  }

  /**
   * Transforms title of a space to NID.
   *
   * @Transform /^NID of space '([A-Za-z0-9_ :@]+)'$/
   */
  public function getNidFromTitle($space_title) {
    $space_title = $this->getStaticTimestamp($space_title);
    $node = $this->getSpace($space_title, FALSE);
    if ($node) {
      return $node->nid;
    }
    else {
      throw new \Exception(sprintf('The space with title %s was not found on the page %s', $space_title, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * Gets the path.
   *
   * @Transform /^Path for space '([A-Za-z0-9_ :@]+)'$/
   */
  public function getPathForSpace($space_title) {
    $space_title = $this->getStaticTimestamp($space_title);
    $node = $this->getSpace($space_title, FALSE);
    if ($node) {
      return 'node/' . $node->nid;
    }
    else {
      throw new \Exception(sprintf('The space with title %s was not found on the page %s', $space_title, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * Gets the space title formatted for select2 results..
   *
   * @Transform /^Select2 value for space '([A-Za-z0-9_ :@]+)'$/
   */
  public function getSelect2ValueForSpace($space_title) {
    $space_title = $this->getStaticTimestamp($space_title);
    $node = $this->getSpace($space_title, FALSE);
    if ($node) {
      return $node->title . ' (' . $node->nid . ')';
    }
    else {
      throw new \Exception(sprintf('The space with title %s was not found on the page %s', $space_title, $this->getSession()->getCurrentUrl()));
    }
  }


  /**
   * Gets the space title formatted for select2 results..
   *
   * @Transform /^Select2 value for group '([A-Za-z0-9_ :@]+)'$/
   */
  public function getSelect2ValueForGroup($group_title) {
    $group_title = $this->getStaticTimestamp($group_title);
    $node = NULL;
    if ($nodes = node_load_multiple(array(), array('title' => $group_title, 'type' => 'oa_group'))) {
      $node = reset($nodes);
    }
    if ($node) {
      return $node->title . ' (' . $node->nid . ')';
    }
    else {
      throw new \Exception(sprintf('The space with title %s was not found on the page %s', $group_title, $this->getSession()->getCurrentUrl()));
    }
  }

  /**
   * Gets the content title formatted for select2 results..
   *
   * @Transform /^Select2 value for content '([A-Za-z0-9_ :@]+)' in '([A-Za-z0-9_ :@]+)'$/
   */
  public function getSelect2ValueForContent($content_title, $space_title) {
    $nid = $this->getNidFromTitleForSpace($space_title);
    $node = $this->getSpace($space_title, FALSE);
    if ($node = node_load($nid)) {
      return $node->title . ' (' . $node->nid . ')';
    }
    else {
      throw new \Exception(sprintf('The space with title %s was not found on the page %s', $space_title, $this->getSession()->getCurrentUrl()));
    }
  }


  /**
   * Gets the timestamp.
   *
   * @Transform /^(([^"]*@timestamp[^"]*)*)$/
   */
  public function getStaticTimestamp($arg) {
    static $time;
    if (!$time) {
      $time = time();
    }
    return str_replace('@timestamp', $time, $arg);
  }

  /**
   * @AfterScenario
   *
   * Clean up users that were created during testing
   */
  public function removeUsers() {
    if(!empty($this->users)){
      foreach ($this->users as $user) {
        $account = user_load_by_name($user->name);
        user_delete($account->uid);
      }
      $this->users = array();
    }
  }

  /**
   * @AfterScenario
   *
   * Clean up tours that were created.
   */
  public function removeTours() {
    foreach (bootstrap_tour_load_all() as $tour) {
      if (strpos($tour->name, 'behat_tour') === 0) {
         entity_delete('bootstrap_tour', $tour->bootstrap_tour_id);
      }
    }
  }

  /**
   * @AfterScenario
   *
   * Restore comment settings
   */
  public function restoreCommentSettings() {
    foreach ($this->comment_settings as $content_type => $value) {
      variable_set('comment_' . $content_type, $value);
    }
  }

  /**
   * @AfterScenario
   *
   * Restore comment settings
   */
  public function restoreCommentParagraphSettings() {
    foreach ($this->comment_paragraph_settings as $content_type => $value) {
      variable_set('oa_related_comment_' . $content_type, $value);
    }
  }

  /**
   * @AfterScenario
   *
   * Remove behat_test content type
   */
  public function removeBehatTestContentType() {
    node_types_rebuild();
    if (node_type_get_type('behat_test')) {
      node_type_delete('behat_test');
    }
    if ($view = views_get_view('behat_test')) {
      views_delete_view($view);
    }
  }

  /**
   * @AfterScenario
   *
   * Remove space type terms
   */
  public function removeSpaceTypeTerms() {
    if ($vocab = taxonomy_vocabulary_machine_name_load('space_type')) {
      if ($terms = taxonomy_term_load_multiple(array(), array('vid' => $vocab->vid))) {
        foreach ($terms as $term) {
          if (strpos($term->name, 'Space Type ') === 0) {
            taxonomy_term_delete($term->tid);
          }
        }
      }
    }
  }

  /**
   * Set comment settings
   */
  public function setCommentSettngs($content_type, $value) {
    $content_type = $this->transformContentTypeName($content_type);
    $current_value = variable_get('comment_' . $content_type, COMMENT_NODE_OPEN);
    if ($current_value != $value) {
      if (!isset($this->comment_settings[$content_type])) {
        $this->comment_settings[$content_type] = $current_value;
      }
      variable_set('comment_' . $content_type, $value);
    }
  }

  /**
   * Set comment settings
   */
  public function setCommentParagraphSettngs($content_type, $value) {
    if (!module_exists('oa_related')) {
      return;
    }
    $content_type = $this->transformContentTypeName($content_type);

    $enabled_related = oa_related_get_content_types();
    $current_value = variable_get('oa_related_comment_' . $content_type, in_array($content_type, $enabled_related));

    if ($value != $current_value) {
      // Record the value for later restoring (only if in database).
      if (!isset($this->comment_paragraph_settings[$content_type])) {
        $this->comment_paragraph_settings[$content_type] = $current_value;
      }
      variable_set('oa_related_comment_' . $content_type, $value);
      drupal_static_reset('oa_comment_content_types_enabled');
      $current_path = getcwd();
      chdir(DRUPAL_ROOT);
      drupal_static_reset('features_get_default');
      features_template_revert();
      chdir($current_path);
      variable_del('menu_rebuild_needed');
    }
  }

  /**
   * Changes from content type name to machine name if is one.
   */
  public function transformContentTypeName($content_type) {
    $names = node_type_get_names();
    // Convert content type name to machine name if passed in.
    return in_array($content_type, $names) ? array_search($content_type, $names) : $content_type;
  }


  /**
   * @BeforeStep
   */
  public function beforeStep()  {
    $this->getSession()->resizeWindow(2280, 1024, 'current');
  }
}
